var _interopRequireDefault=require("@babel/runtime/helpers/interopRequireDefault");Object.defineProperty(exports,"__esModule",{value:true});exports.checkAndMigrateProject=void 0;var _fs=_interopRequireDefault(require("fs"));
var _path=_interopRequireDefault(require("path"));

var _prompt=require("../systemTools/prompt");
var _logger=require("../systemTools/logger");
var _fileutils=require("../systemTools/fileutils");
var _configParser=require("../configTools/configParser");
var _cleaner=require("../systemTools/cleaner");
var _projectParser=require("./projectParser");

var checkAndMigrateProject=function checkAndMigrateProject(c){return new Promise(function(resolve,reject){
var prjDir=c.paths.project.dir;


var paths={
project:prjDir,
globalConfig:_path.default.join(c.paths.GLOBAL_RNV_DIR,'config.json'),
config:_path.default.join(prjDir,'rnv-config.json'),
package:_path.default.join(prjDir,'package.json'),
plugins:_path.default.join(prjDir,'projectConfig/plugins.json'),
permissions:_path.default.join(prjDir,'projectConfig/permissions.json'),
appConfigs:_path.default.join(prjDir,'appConfigs')};


try{
paths.appConfigDirs=(0,_configParser.listAppConfigsFoldersSync)(c).map(function(v){return prjDir+"/appConfigs/"+v+"/config.json";});
}catch(e){
(0,_logger.logWarning)(e);
}

if(_fs.default.existsSync(paths.config)){
(0,_prompt.askQuestion)('Your project has been created with previous version of ReNative. Do you want to migrate it to new format? Backing up project is recommended! To Proceed type: (y)').
then(function(v){
if(v==='y'){
c.program.reset=true;
_migrateProject(c,paths).
then(function(){return(0,_cleaner.cleanProjectModules)(c);}).
then(function(){return(0,_projectParser.configureNodeModules)(c);}).
then(function(){return resolve();}).
catch(function(e){return reject(e);});
return;
}
return resolve();
}).
catch(function(e){return reject(e);});
}else{
return resolve();
}
});};exports.checkAndMigrateProject=checkAndMigrateProject;

var PATH_PROPS=[
'globalConfigFolder',
'appConfigsFolder',
'platformTemplatesFolder',
'entryFolder',
'platformAssetsFolder',
'platformBuildsFolder',
'projectPlugins',
'projectConfigFolder'];



var _migrateProject=function _migrateProject(c,paths){return new Promise(function(resolve,reject){
(0,_logger.logTask)('MIGRATION STARTED');

if(!_fs.default.existsSync(c.paths.GLOBAL_RNV_CONFIG)){
if(_fs.default.existsSync(paths.globalConfig)){
(0,_fileutils.copyFileSync)(paths.globalConfig,c.paths.GLOBAL_RNV_CONFIG);
}
}

var files={
config:(0,_fileutils.readObjectSync)(paths.config),
package:(0,_fileutils.readObjectSync)(paths.package),
plugins:(0,_fileutils.readObjectSync)(paths.plugins),
permissions:(0,_fileutils.readObjectSync)(paths.permissions)};


(0,_logger.logDebug)("paths to migrate: \n "+paths);

var newConfig={};

if(files.package){
newConfig.projectName=files.package.name;
}

if(files.config){
newConfig.defaults={};

if(files.config.defaultProjectConfigs){
newConfig.defaults=(0,_fileutils.mergeObjects)(newConfig.defaults,files.config.defaultProjectConfigs);
}
newConfig.currentTemplate=newConfig.defaults.template||'renative-template-hello-world';

newConfig.paths={};
PATH_PROPS.forEach(function(v){
if(files.config[v]){
newConfig.paths[v]=files.config[v];
}
});

if(files.config.defaultPorts){
newConfig.defaults.ports=files.config.defaultPorts;
}

if(files.config.crypto){
newConfig.crypto=files.config.crypto;
}
}

if(!newConfig.platforms)newConfig.platforms={};

if(files.plugins){
newConfig.plugins=files.plugins.plugins;

if(files.plugins.android){
newConfig.platforms.android=files.plugins.android;
}
if(files.plugins.ios){
newConfig.platforms.ios=files.plugins.ios;
}
}

if(files.permissions){
newConfig.permissions=files.permissions.permissions;
}

var pathsToDelete=[
paths.config,
paths.plugins,
paths.permissions].
concat(paths.appConfigDirs);

paths.appConfigDirs.forEach(function(v){
if(_fs.default.existsSync(v)){
(0,_fileutils.copyFileSync)(v,v.replace('/config.json','/renative.json'));
}
});

(0,_fileutils.writeObjectSync)(c.paths.project.config,newConfig);

(0,_logger.logDebug)("Paths to delete, "+pathsToDelete.join('\n'));

(0,_fileutils.removeFilesSync)(pathsToDelete);

(0,_logger.logSuccess)('Migration Complete!');

resolve();
});};